class OriginalLicensesController < ApplicationController
  layout 'test' if Pettanr::TestLayout
  before_filter :authenticate_admin!, :only => [:list, :browse, :new, :edit, :create, :update, :destroy, :import]

  private
  
  def set_data(file)
    if file.respond_to?(:read)
      file.read
    else
      file
    end
  end
  
  public
  
  # GET /original_licenses
  # GET /original_licenses.json
  def index
    @page = OriginalLicense.page params[:page]
    @page_size = OriginalLicense.page_size params[:page_size]
    @original_licenses = OriginalLicense.list({}, @page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @original_licenses.to_json(OriginalLicense.list_json_opt) }
    end
  end

  # GET /original_licenses/1
  # GET /original_licenses/1.json
  def show
    @original_license = OriginalLicense.show(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @original_license.to_json(OriginalLicense.show_json_include_opt) }
    end
  end

  def list
    @original_licenses = OriginalLicense.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @original_licenses }
    end
  end

  def browse
    @original_license = OriginalLicense.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @original_license }
    end
  end

  # GET /original_licenses/new
  # GET /original_licenses/new.json
  def new
    @original_license = OriginalLicense.new
    @original_license.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @original_license }
    end
  end

  # GET /original_licenses/1/edit
  def edit
    @original_license = OriginalLicense.show(params[:id])
    respond_to do |format|
      format.html
      format.js
    end
  end

  # POST /original_licenses
  # POST /original_licenses.json
  def create
    @original_license = OriginalLicense.new
    @original_license.supply_default
    @original_license.attributes = params[:original_license]
    respond_to do |format|
      if @original_license.store
        format.html { redirect_to @original_license, notice: 'Original license was successfully created.' }
        format.json { render json: @original_license, status: :created, location: @original_license }
      else
        format.html { render action: "new" }
        format.json { render json: @original_license.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /original_licenses/1
  # PUT /original_licenses/1.json
  def update
    @original_license = OriginalLicense.show(params[:id])
    @original_license.attributes = params[:original_license]

    respond_to do |format|
      if @original_license.store 
        format.html { redirect_to @original_license, notice: 'Original license was successfully updated.' }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @original_license.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /original_licenses/1
  # DELETE /original_licenses/1.json
  def destroy
    @original_license = OriginalLicense.find(params[:id])
    @original_license.destroy

    respond_to do |format|
      format.html { redirect_to original_licenses_url }
      format.json { head :ok }
    end
  end
  
  def import
    @data = set_data params[:file]
    respond_to do |format|
      @errors = OriginalLicense.import @data
      if @errors and @errors.empty?
        format.html { redirect_to :action => :list }
        format.json { render text: 'ok', status: :created }
      else
        format.html { render action: "result" }
        format.json { render json: @errors, status: :unprocessable_entity }
      end
    end
  end

end
