# -*- encoding: utf-8 -*-
#ライセンス
require 'spec_helper'

describe License do
  before do
  end
  describe '検証に於いて' do
    before do
      @l = Factory.build :license
    end
    
    it 'オーソドックスなデータなら通る' do
      @l.should be_valid
    end
    
    context 'nameを検証するとき' do
      it 'nullなら失敗する' do
        @l.name = ''
        @l.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @l.name = 'a'*51
        @l.should_not be_valid
      end
    end
    context 'urlを検証するとき' do
      it 'nullなら失敗する' do
        @l.url = ''
        @l.should_not be_valid
      end
      it '201文字以上なら失敗する' do
        @l.url = 'a'*201
        @l.should_not be_valid
      end
      it '重複していたら失敗する' do
        lc = Factory :license
        @l.should_not be_valid
      end
      it 'url形式でなら失敗する' do
        @l.url = ''
        pending
      end
    end
  end
  
  describe '対象ライセンスの取得に於いて' do
    before do
      @lc = Factory :license
    end
    context 'urlが一致するライセンスがないとき' do
      it '新規ライセンスを準備して返す' do
        cl = Factory.build(:common_license, :url => 'http://domain.no')
        r = License.update_license cl
        r.should be_a_new License
      end
    end
    context 'urlが一致するライセンスがあるとき' do
      it '該当ライセンスを返す' do
        r = License.update_license @lc
        r.is_a?(License).should be_true
        r.should_not be_a_new License
        r.url.should eq @lc.url
        r.name.should eq @lc.name
      end
    end
    #コモンライセンスとオリジナルライセンスをまたいだUrl重複チェックはここでやるよりない
    #コモンライセンスが新規オブジェクトなのにライセンスが取得できるのは、
    #そのUrlがオリジナルライセンスから登録されているということ
    context '新規でユニークチェックするとき' do
      it 'ライセンスの全体エラーに重複メッセージを入れて返す' do
        cl = Factory.build(:common_license, :url => 'http://domain.no')
        License.stub(:find_by_url).with(any_args).and_return(@lc)
        r = License.update_license cl
        r.errors[:base].should_not be_empty
      end
    end
  end
  
  #作成が
  describe '単体取得に於いて' do
    before do
      @lcl = Factory :license
      @cl = Factory :common_license, :license_id => @lcl.id
      @lol = Factory :license, :url => 'http://test.ptn/10'
      @ol = Factory :original_license, :license_id => @lol.id, :url => 'http://test.ptn/10'
    end
    it '指定のコマを返す' do
      l = License.show @lcl.id
      l.should eq @lcl
    end
  end
  describe '関連テーブルプションに於いて' do
    context 'オプションがないとき' do
      it 'コモンライセンスとオリジナルライセンスを含んでいる' do
        r = License.show_include_opt
        r.should eq [:common_license, :original_license]
      end
    end
    context 'オプションで原画を含ませたとき' do
      it 'コモンライセンスとオリジナルライセンスと原画データを含んでいる' do
        r = License.show_include_opt(:include => :original_picture)
        r.should eq [:common_license, :original_license, :original_picture]
      end
    end
  end
  describe 'json単体出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = License.show_json_include_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = License.show_json_include_opt[:include]
      r.should have(2).items
    end
    it 'コモンライセンスを含んでいる' do
      r = License.show_json_include_opt[:include]
      r.has_key?(:common_license).should be_true
    end
    it 'オリジナルライセンスを含んでいる' do
      r = License.show_json_include_opt[:include]
      r.has_key?(:original_license).should be_true
    end
  end
  describe '一覧取得に於いて' do
    before do
      @lcl = Factory :license, :name => 'peta2.0'
      @cl = Factory :common_license, :license_id => @lcl.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        License.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        License.page().should eq 1
      end
      it '0以下の場合は1になる' do
        License.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        License.page_size('7').should eq 7
      end
      it 'nilの場合はLicense.default_page_sizeになる' do
        License.page_size().should eq License.default_page_size
      end
      it '0以下の場合はLicense.default_page_sizeになる' do
        License.page_size('0').should eq License.default_page_size
      end
      it 'License.max_page_sizeを超えた場合はLicense.max_page_sizeになる' do
        License.page_size('1000').should eq License.max_page_size
      end
    end
    it 'リストを返す' do
      pl = License.list
      pl.should eq [@lcl]
    end
    it '名前順で並んでいる' do
      @lol = Factory :license, :name => 'peta1.0', :url => 'http://test.ptn/10'
      @ol = Factory :original_license, :license_id => @lol.id, :name => 'peta1.0', :url => 'http://test.ptn/10'
      l = License.list
      l.should eq [@lol, @lcl]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @lol2 = Factory :license, :name => 'peta2.1', :url => 'http://test.ptn/21'
        @ol2 = Factory :original_license, :license_id => @lol2.id, :name => 'peta2.1', :url => 'http://test.ptn/21'
        @lol3 = Factory :license, :name => 'peta2.2', :url => 'http://test.ptn/22'
        @ol3 = Factory :original_license, :license_id => @lol3.id, :name => 'peta2.2', :url => 'http://test.ptn/22'
        @lol4 = Factory :license, :name => 'peta2.3', :url => 'http://test.ptn/23'
        @ol4 = Factory :original_license, :license_id => @lol4.id, :name => 'peta2.3', :url => 'http://test.ptn/23'
        @lol5 = Factory :license, :name => 'peta2.4', :url => 'http://test.ptn/24'
        @ol5 = Factory :original_license, :license_id => @lol5.id, :name => 'peta2.4', :url => 'http://test.ptn/24'
        License.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        l = License.list
        l.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #名前順で並んでいる
        l = License.list( {}, 1)
        l.should eq [@lcl, @lol2]
      end
      it 'page=2なら中間2件を返す' do
        l = License.list({}, 2)
        l.should eq [@lol3, @lol4]
      end
      it 'page=3なら先頭1件を返す' do
        l = License.list({}, 3)
        l.should eq [@lol5]
      end
    end
  end
  describe 'list関連テーブルプションに於いて' do
    it 'includeキーを含んでいる' do
      r = License.list_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = License.list_opt[:include]
      r.should have(2).items
    end
    it 'コモンライセンスを含んでいる' do
      r = License.list_opt[:include]
      r.has_key?(:common_license).should be_true
    end
    it 'オリジナルライセンスを含んでいる' do
      r = License.list_opt[:include]
      r.has_key?(:original_license).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = License.list_json_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = License.list_json_opt[:include]
      r.should have(2).items
    end
    it 'コモンライセンスを含んでいる' do
      r = License.list_json_opt[:include]
      r.has_key?(:common_license).should be_true
    end
    it 'オリジナルライセンスを含んでいる' do
      r = License.list_json_opt[:include]
      r.has_key?(:original_license).should be_true
    end
  end
  
end
